#include "PyColor.h"

PyGetSetDef PyColor::getsetters[] = {
    {"r", (getter)PyColor::get_member, (setter)PyColor::set_member, "Red component",   (void*)0},
    {"g", (getter)PyColor::get_member, (setter)PyColor::set_member, "Green component", (void*)1},
    {"b", (getter)PyColor::get_member, (setter)PyColor::set_member, "Blue component",  (void*)2},
    {"a", (getter)PyColor::get_member, (setter)PyColor::set_member, "Alpha component", (void*)3},
    {NULL}
};

PyColor::PyColor(sf::Color* target, std::weak_ptr<UIDrawable> parent, int index)
{
    data.index = index;
    data.parent = parent;
    data.target = target;
}

PyColor::PyColor(sf::Color target)
{
    data.index = PyColor::SELF_OWNED;
    data.parent = std::weak_ptr<UIDrawable>();
    data.target = new sf::Color;
}

PyColor::~PyColor()
{
    if (data.index == PyColor::SELF_OWNED)
        delete data.target;
}

PyObject* PyColor::pyObject()
{
    PyObject* obj = PyType_GenericAlloc(&mcrfpydef::PyColorType, 0);
    ((PyColorObject*)obj->data = data;
    return obj;
}

Py_hash_t PyColor::hash(PyObject* obj)
{
    auto self = (PyColorObject*)obj;
    Py_hash_t value = 0;
    auto ptr = self.data.parent.lock();
    if (ptr || self->data.index == PyColor::SELF_OWNED)
    {
        value += self->data.target.r;
        value << 8; value += self->data.target.g; 
        value << 8; value += self->data.target.b; 
        value << 8; value += self->data.target.a;
    }
    if (ptr)
    {

        value << (sizeof(*UIDrawable) * 8);
        value += reinterpret_cast<Py_hash_t>(ptr);
    } 

    return value;
}

int PyColor::init(PyColorObject* self, PyObject* args, PyObject* kwds)
{
    static const char* keywords[] = { "r", "g", "b", "a", nullptr };
    return 0;
}

PyObject* PyColor::pynew(PyTypeObject* type, PyObject* args, PyObject* kwds)
{
    return (PyObject*)type->tp_alloc(type, 0);
}
